<?php

namespace InSegment\ApiCore\Interfaces;

interface SADConsumer
{
    /**
     * In the first part of the cycle, SAD instance will process attributes of the models and call this method on each
     * 
     * @param mixed $attribute
     */
    public function acceptAttribute($attribute);
    
    /**
     * In the second part of the cycle, SAD instance will process separately downstream attributes of the models and call
     * this method on each. By default, there are none downstream attributes - the consumer should call
     * methods SourceAttributeDownstream::addDownStream to affect the cycle
     * 
     * @param mixed $downstream
     */
    public function acceptDownstream($downstream);
    
    /**
     * Before attributes and downstream attributes are processed, this method will be called by SAD instance to
     * start processing of Model. By default, SAD instance will only process models from the initial array -  to add
     * new models, the consumer should call method SourceAttributeDownstream::newSource, which will affect the cycle
     * 
     * @param mxied $model
     */
    public function acceptModel($model);

    /**
     * After attributes and downstream attributes are processed, this method will be called by SAD instance to
     * start processing of Model. By default, SAD instance will only process models from the initial array -  to add
     * new models, the consumer should call method SourceAttributeDownstream::newSource, which will affect the cycle
     * 
     * @param mxied $model
     */
    public function saveModel($model);
}
